package fxtaidegalleria;

import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.GridPane;
import javafx.scene.text.Font;
import taidegalleria.*;

import java.io.PrintStream;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

/**
 * @author lainejee
 * @version 26.1.2025
 */
public class TaidegalleriaGUIController implements Initializable {

    @FXML private ListChooser<Taiteilija> chooserTaiteilijat;
    @FXML private ScrollPane panelTaiteilija;


    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        alusta();
    }


    @FXML
    private void handleAvaa() {
        avaa();
    }


    @FXML private void handleTallenna() {
        Dialogs.showMessageDialog("Ei osata vielä toteuttaa");
    }


    @FXML private void handleSulje() {
        Platform.exit();
    }


    /**
     * Aukaisee valikon jossa voi lisätä uuden taiteilijan.
     * Vielä ei ole muuta toiminnallisuutta.
     */
    @FXML private void handleUusiTaiteilija() {
        // ModalController.showModal(TaidegalleriaGUIController.class.getResource("LisaaTaiteilijaGUIView.fxml"), "Taiteilija", null, "");
        uusiTaiteilija();
    }


    /**
     * Aukaisee valikon jossa voi lisätä uuden teoksen.
     * Vielä ei ole muuta toiminnallisuutta.
     */
    @FXML private void handleUusiTeos() {
        // ModalController.showModal(TaidegalleriaGUIController.class.getResource("LisaaTeosGUIView.fxml"), "Taideteos", null, "");
        uusiTaideteos();
    }


    @FXML private void handlePoistaTaiteilija() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa taiteilijaa");
    }


    @FXML private void handleTietoja() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa taiteilijaa");
    }


    @FXML private void handlePoistaTeos() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa teosta");
    }


    @FXML private void handleTietokanta() {
        Dialogs.showMessageDialog("Ei osata vielä toteuttaa");
    }


    @FXML private void handleGalleria() {
        Dialogs.showMessageDialog("Ei osata vielä toteuttaa");
    }


    @FXML void keyReleasedTaiteilija(KeyEvent event) {
        Dialogs.showMessageDialog("Ei osata vielä toteuttaa");
    }


    @FXML void keyReleasedTeos(KeyEvent event) {
        Dialogs.showMessageDialog("Ei osata vielä toteuttaa");
    }


    //============================================================================
    // Tämän viivan alle tulee asiat jotka eivät liity JavaFX puuhasteluun.

    private Tietokanta tietokanta;
    private TextArea areaTaiteilija = new TextArea(); // TODO: Poistetaan lopuksi.


    /**
     * Tyhjentää listan taiteilijoista.
     */
    private void alusta() {
        panelTaiteilija.setContent(areaTaiteilija);
        areaTaiteilija.setFont(new Font("Courier New", 12));
        panelTaiteilija.setFitToHeight(true);

        chooserTaiteilijat.clear();
        chooserTaiteilijat.addSelectionListener(e -> naytaTaiteilija());
    }


    /**
     * Kysytään tietokannan nimeä ja luetaan se.
     * @return true jos luku onnistuu, false jos ei
     */
    public boolean avaa() {
        String uusinimi = TietokannanNimiController.kysyNimi(null, "suosikit");
        if (uusinimi == null) return false;
        // lueTiedosto(uusinimi);
        return true;
    }


    /**
     *
     */
    private void naytaTaiteilija() {
        Taiteilija taiteilijaKohdalla = chooserTaiteilijat.getSelectedObject();

        if (taiteilijaKohdalla == null) return;

        areaTaiteilija.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaTaiteilija)) {
            taiteilijaKohdalla.tulosta(os);
            List<Taideteos> taideteokset = tietokanta.annaTaideteokset(taiteilijaKohdalla);
            for (Taideteos teos : taideteokset)
                teos.tulosta(os);
        }

    }


    /**
     * Haetaan taiteilijat uudelleen
     * @param taiteilijaId mikä taiteilija valitaan aktiiviseksi
     */
    private void hae(int taiteilijaId) {
        chooserTaiteilijat.clear();

        int index = 0;
        for (int i = 0; i < tietokanta.getTaiteilijoita(); i++) {
            Taiteilija taiteilija = tietokanta.annaTaiteilija(i);
            if (taiteilija.getId() == taiteilijaId) index = i;
            chooserTaiteilijat.add(taiteilija.getNimi(), taiteilija);
        }
        chooserTaiteilijat.setSelectedIndex(index);
    }

    /**
     *
     */
    private void uusiTaiteilija() {
        Taiteilija uusi = new Taiteilija();
        uusi.rekisteroi(); // TODO: Oikeasti rekisteröintiä ei tehdä ennen kuin käyttäjä on painanut OK:ta!
        uusi.tayta(); // TODO: Tähän possautetaan auki taiteilijan tietojen syöttämis dialogi! Jos painaa OK, niin lisätään jäsen.. jne.
        try {
            tietokanta.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden taiteilijan luomisessa: " + e.getMessage());
            return;
        }
        hae(uusi.getId());
    }


    private void uusiTaideteos() {
        Taiteilija taiteilijaKohdalla = chooserTaiteilijat.getSelectedObject();
        if ( taiteilijaKohdalla == null ) Dialogs.showMessageDialog("Lisää ensin taiteilija.");
        Taideteos teos = new Taideteos();
        teos.rekisteroi();
        teos.vastaaTestiTaideteos(taiteilijaKohdalla.getId()); // TODO: Pitäisi näyttää dialogi jossa kysytään tiedot.
        tietokanta.lisaa(teos);
        hae(taiteilijaKohdalla.getId());


    }

    /**
     * Tulostaa jäsenen tiedot
     * @param os tietovirta johon tulostetaan
     * @param taiteilija tulostettava jäsen
     */
    public void tulosta(PrintStream os, final Taiteilija taiteilija) {
        os.println("----------------------------------------------");
        taiteilija.tulosta(os);
        os.println("----------------------------------------------");
        List<Taideteos> taideteokset = tietokanta.annaTaideteokset(taiteilija);
        for (Taideteos har:taideteokset)
            har.tulosta(os);
    }



    /**
     * Asetetaan käytettävä tietokanta.
     * @param tietokanta
     */
    public void setTietokanta(Tietokanta tietokanta) {
        this.tietokanta = tietokanta;
    }


}