package taidegalleria;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * @author lainejee
 * @version 1.0 19.3.2025
 * |------------------------------------------------------------------------|
 * | Luokan nimi:   Taideteos                           | Avustajat:        |
 * |-------------------------------------------------------------------------
 * | Vastuualueet:                                      |                   |
 * |                                                    |                   |
 * | (- Ei tiedä tietokannasta eikä käyttöliittymästä.) |                   |
 * |  - Tietää taideteoksen kentät (nimi, vuosi, jne.)  |                   |
 * |  - Osaa tarkistaa tietyn kentän oikeellisuuden.    |                   |
 * |  - Osaa muuttaa 2|t003|Nighthawks|..|-merkkijonon  |                   |
 * |    taideteoksen tiedoiksi.                         |                   |
 * |  - Osaa antaa merkkijonona indeksin kentän tieodt. |                   |
 * |  - Osaa laittaa merkkijonon indeksin kentäksi.     |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |-------------------------------------------------------------------------
 */

public class Taideteos {

    private int taiteilijaTunnus = 0;
    private int teosTunnus = 0;
    private String teoksenNimi = "";
    private String vuosiluku = "0";
    private String tyylisuunta = "";
    private String tekniikka = "";
    private String aihe = "";
    private String maisemalokaatio = "";
    private String kuvapolku = "";

    public static int seuraavaTeosId = 1;


    /**
     * Muodostaa taideteoksen.
     */
    public Taideteos() {
        // Ei tarvita vielä.
    }


    /**
     * Alustetaan tietyn taiteilijan taideteos.
     * @param taiteilijaTunnus taiteilijan tunnusnumero
     */
    public Taideteos(int taiteilijaTunnus) {
        this.taiteilijaTunnus = taiteilijaTunnus;
    }


    /**
     * Apumetodi jolla testataan taideteoksen täyttämistä.
     */
    public void vastaaTestiTaideteos(int nro) {
        taiteilijaTunnus = nro;
        teoksenNimi = "Hiroshigen maalaus";
        vuosiluku = "1800-luvulla";
        tyylisuunta = "Ukiyo-e";
        tekniikka = "Woodblock";
        aihe = "maisema";
        maisemalokaatio = "35.71492187217169,139.79671966985129";
    }


    /**
     * Tulostetaan taiteilijan tiedot.
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(taiteilijaTunnus + " " + String.format("%03d", teosTunnus) + " " + teoksenNimi);
        out.println("Valmistunut: " + vuosiluku + ". " + "Tyylisuunta: " + tyylisuunta + " Tekniikka: " + tekniikka + " Aihe: " + aihe);
        out.println("Maisema löytyy lokaatiosta: " + maisemalokaatio);
    }


    /**
     * Tulostetaan taiteilijan tiedot. Ei tarvita vielä.
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }


    /**
     * Antaa taiteilijalle seuraavan id:n.
     * @return taiteilijan uusi tunnusId
     * @example
     * <pre name="test">
     *     Taiteilija a1 = new Taiteilija();
     *     a1.getId() === 0;
     *     a1.rekisteroi();
     *     Taiteilija a2 = new Taiteilija();
     *     a2.rekisteroi();
     *     int n1 = a1.getId();
     *     int n2 = a2.getId();
     *     n1 === n2-1;
     * </pre>
     */
    public int rekisteroi() {
        teosTunnus = seuraavaTeosId;
        seuraavaTeosId++;
        return teosTunnus;
    }


    /**
     * Palauttaa teoksen id:n. Ei käytetä vielä.
     * @return teoksen id
     */
    public int getTeosTunnus() {
        return teosTunnus;
    }


    /**
     * Palauttaa mille taiteilijalle teos kuuluu.
     * @return taiteilijan tunnuksen
     */
    public int getTaiteilijaTunnus() {
        return taiteilijaTunnus;
    }


    /**
     * Testaan taideteos-luokkaa.
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taideteos t001 = new Taideteos();
        Taideteos t002 = new Taideteos();

        t001.rekisteroi();
        t002.rekisteroi();

        t001.tulosta(System.out);
        t002.tulosta(System.out);

        t001.vastaaTestiTaideteos(1);
        t002.vastaaTestiTaideteos(2);

        t001.tulosta(System.out);
        t002.tulosta(System.out);

    }

}
