package taidegalleria;

/**
 * @author lainejee
 * @version 1.0 24.2.2025
 * |------------------------------------------------------------------------|
 * | Luokan nimi:   Taiteilijat                         | Avustajat:        |
 * |-------------------------------------------------------------------------
 * | Vastuualueet:                                      |                   |
 * |                                                    | - Taiteilija      |
 * |  - Pitää yllä varsinaista rekisteriä               |                   |
 * |    taiteilijoista, eli osaa lisätä ja poistaa      |                   |
 * |    taiteilijan.                                    |                   |
 * |  - Lukee ja kirjoittaa taiteilijan tiedostoon.     |                   |
 * |  - Osaa etsiä ja lajitella.                        |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |-------------------------------------------------------------------------
 */

public class Taiteilijat {

    private static final int MAX_TAITEILIJAT = 5;
    int lkm = 0;
    private final Taiteilija[] alkiot;


    /**
     * Muodostetaan alustava taulukko.
     */
    public Taiteilijat() {
        this.alkiot = new Taiteilija[MAX_TAITEILIJAT];
    }


    /**
     * Lisää uuden taiteilijan tietorakenteeseen.
     * @param taiteilija lisättävän taiteilijan viite
     * @throws SailoException jos tietorakenne on täynnä
     * @example
     * <pre name="test">
     *     #THROWS SailoException
     *     Taiteilijat suosikit = new Taiteilijat();
     *     Taiteilija a1 = new Taiteilija(), a2 = new Taiteilija();
     *     suosikit.getLkm() === 0;
     *     suosikit.lisaa(a1); suosikit.getLkm() === 1;
     *     suosikit.lisaa(a2); suosikit.getLkm() === 2;
     *     suosikit.lisaa(a1); suosikit.getLkm() === 3;
     *     suosikit.anna(0) === a1;
     *     suosikit.anna(1) === a2;
     *     suosikit.anna(2) === a1;
     *     suosikit.anna(1) == a1 === false;
     *     suosikit.anna(1) == a2 === true;
     *     suosikit.anna(3) === a1; #THROWS IndexOutOfBoundsException
     *     suosikit.lisaa(a1); suosikit.getLkm() === 4;
     *     suosikit.lisaa(a1); suosikit.getLkm() === 5;
     *     suosikit.lisaa(a1); #THROWS SailoException
     * </pre>
     */
    public void lisaa(Taiteilija taiteilija) throws SailoException {
        if (lkm >= alkiot.length) throw new SailoException("Liikaa alkioita");
        alkiot[lkm] = taiteilija;
        lkm++;
    }


    /**
     * Palauttaa viitteen i:teen taiteilijaan.
     * @param i taiteilijan indeksi
     * @return viite taiteilijaan, i:n kohdalta
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella
     */
    public Taiteilija anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || this.lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }


    /**
     * Palauttaa tietokannan taiteilijoiden lukumäärän.
     * @return taiteilijoiden lukumäärä
     */
    public int getLkm() {
        return lkm;
    }


    /**
     * Testataan taiteilijat-luokkaa.
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taiteilijat suosikit = new Taiteilijat();

        Taiteilija hiroshige = new Taiteilija();
        Taiteilija edward = new Taiteilija();

        hiroshige.rekisteroi();
        hiroshige.tayta();
        edward.rekisteroi();
        edward.tayta();

        try {
            suosikit.lisaa(hiroshige);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
            suosikit.lisaa(edward);
        } catch (SailoException e) {
            System.err.println(e.getMessage());
        }

        System.out.println("================== SUOSIKIT TESTI ==================");

        for (int i = 0; i < suosikit.getLkm(); i++) {
            Taiteilija taiteilija = suosikit.anna(i);
            System.out.println("Taiteilijan indeksi: " + i);
            taiteilija.tulosta(System.out);
        }

    }

}
